@@ -4,7 +4,7 @@ import argparse
 import copy
 import json
 import logging
-from typing import AsyncIterable
+from typing import Any, AsyncIterable
 
 from fastapi import FastAPI, HTTPException, Request
 from fastapi.exceptions import RequestValidationError
@@ -15,7 +15,7 @@ from starlette.middleware.base import BaseHTTPMiddleware
 from fastapi_poe.types import (
     ContentType,
     QueryRequest,
-    RawRequest,
+    ReportErrorRequest,
     ReportFeedbackRequest,
     SettingsRequest,
     SettingsResponse,
@@ -63,6 +63,10 @@ class PoeHandler:
         """Override this to record feedback from the user."""
         pass
 
+    async def on_error(self, error_request: ReportErrorRequest) -> None:
+        """Override this to record errors from the Poe server."""
+        logger.error(f"Error from Poe server: {error_request}")
+
     # Helpers for generating responses
 
     @staticmethod
@@ -112,6 +116,12 @@ class PoeHandler:
         await self.on_feedback(feedback_request)
         return JSONResponse({})
 
+    async def handle_report_error(
+        self, error_request: ReportErrorRequest
+    ) -> JSONResponse:
+        await self.on_error(error_request)
+        return JSONResponse({})
+
     async def handle_settings(self, settings_request: SettingsRequest) -> JSONResponse:
         settings = await self.get_settings(settings_request)
         return JSONResponse(settings.dict())
@@ -134,13 +144,21 @@ def run(handler: PoeHandler) -> None:
     app.add_exception_handler(RequestValidationError, exception_handler)
 
     @app.post("/")
-    async def run_inner(conversation_request: RawRequest):
-        if conversation_request.type == "query":
-            return EventSourceResponse(handler.handle_query(conversation_request))
-        elif conversation_request.type == "settings":
-            return handler.handle_settings(conversation_request)
-        elif conversation_request.type == "report_feedback":
-            return handler.handle_report_feedback(conversation_request)
+    async def run_inner(request: dict[str, Any]):
+        if request["type"] == "query":
+            return EventSourceResponse(
+                handler.handle_query(QueryRequest.parse_obj(request))
+            )
+        elif request["type"] == "settings":
+            return await handler.handle_settings(SettingsRequest.parse_obj(request))
+        elif request["type"] == "report_feedback":
+            return await handler.handle_report_feedback(
+                ReportFeedbackRequest.parse_obj(request)
+            )
+        elif request["type"] == "report_error":
+            return await handler.handle_report_error(
+                ReportErrorRequest.parse_obj(request)
+            )
         else:
             raise HTTPException(status_code=501, detail="Unsupported request type")
 
